#if !defined(__AVM_NET_CFG_ATHEROS_HW214_H_)
#define __AVM_NET_CFG_ATHEROS_HW214_H_

#include <avmnet_module.h>
#include <avmnet_config.h>
#include "../switch/atheros/atheros_mac.h"
#include "../phy/avmnet_ar803x.h"
#include "../phy/avmnet_ar8326.h"

#include <atheros.h>

extern avmnet_module_t hw214_gmac0, hw214_module_eth0;

avmnet_device_t avmnet_hw214_avm_device_0 ____cacheline_aligned =
{
   .device            = NULL,
   .device_name       = "eth0",
   .external_port_no  = 0,
   .device_ops        = {
                          /*--- .ndo_get_stats        = athr_gmac_get_stats, ---*/
                          .ndo_open             = avmnet_netdev_open,
                          .ndo_stop             = avmnet_netdev_stop,
                          /*--- .ndo_do_ioctl         = athr_gmac_do_ioctl, ---*/
                          .ndo_tx_timeout       = athr_gmac_tx_timeout,
                          .ndo_start_xmit       = athr_gmac_hard_start
                        },
   .mac_module        = &hw214_gmac0,
   .vlanID            = 0, // GMAC0
   .sizeof_priv       = sizeof(avmnet_netdev_priv_t),
   .device_setup      = athr_gmac_setup_eth,
   .device_setup_priv = athr_gmac_setup_eth_priv,
};

avmnet_device_t *avmnet_hw214_avm_devices[] = {
    &avmnet_hw214_avm_device_0,
}; 

avmnet_module_t avmnet_hw214 ____cacheline_aligned =
{
    .name           = "athmac",
    .type           = avmnet_modtype_switch,
    .priv           = NULL,
    .initdata       = { .mac = { .flags = 0 }},

    .init           = athmac_init,
    .setup          = athmac_setup,
    .exit           = athmac_exit,
    .poll           = athmac_poll,
    
    .parent         = NULL,
    .num_children   = 1,
    .children       = { &hw214_gmac0 }
};

avmnet_module_t hw214_gmac0 ____cacheline_aligned =
{
    .name           = "gmac0",
    .type           = avmnet_modtype_switch,
    .priv           = NULL,
    .initdata.mac   = { .flags = AVMNET_CONFIG_FLAG_BASEADDR | 
                                 AVMNET_CONFIG_FLAG_IRQ | 
                                 AVMNET_CONFIG_FLAG_BASEADDR_MDIO,
                        .base_addr = ATH_GE0_BASE, 
                        .base_mdio = ATH_GE0_BASE,
                        .irq = ATH_CPU_IRQ_GE0,
                        .mac_nr = 0 
                      },

    .init           = athmac_gmac_init,
    .setup          = athmac_gmac_setup,
    .exit           = athmac_gmac_exit,
    .poll           = athmac_poll,

    .reg_read       = athmac_reg_read,
    .reg_write      = athmac_reg_write,
    .lock           = athmac_lock,
    .unlock         = athmac_unlock,
    .status_changed = athmac_status_changed,
    .set_status     = athgmac_set_status,
    .setup_irq      = athmac_setup_irq,

    .reinit         = athgmac_reinit,   /*--- wird beim AR8033 für fast_reset bei Linkup genutzt ---*/

    .parent         = &avmnet_hw214,
    .num_children   = 1,
    .children       = { &hw214_module_eth0 },
};

avmnet_module_t hw214_module_eth0 ____cacheline_aligned =
{
    .name           = "ar8035",
    .device_id      = &avmnet_hw214_avm_device_0,
    .type           = avmnet_modtype_phy,
    .priv           = NULL,
    .initdata.phy   = { .flags =   AVMNET_CONFIG_FLAG_RESET 
                                 | AVMNET_CONFIG_FLAG_PHY_GBIT
                                 | AVMNET_CONFIG_FLAG_IRQ
                                 | AVMNET_CONFIG_FLAG_MDIOADDR
                                 | AVMNET_CONFIG_FLAG_TX_DELAY,
                                 .mdio_addr = 0, 
                                 .tx_delay = 0x2d47,
                                 .irq = ATH_GPIO_IRQn(18),
                                 .reset = 17
                      },

    .init               = avmnet_ar803x_init,            
    .setup              = avmnet_ar803x_setup,           
    .exit               = avmnet_ar803x_exit,            
    .reg_read           = avmnet_ar803x_reg_read,        
    .reg_write          = avmnet_ar803x_reg_write,       
    .lock               = avmnet_ar803x_lock,            
    .unlock             = avmnet_ar803x_unlock,          
    .status_changed     = avmnet_ar803x_status_changed,  
    .poll               = avmnet_ar803x_status_poll,     
    .setup_irq          = avmnet_ar803x_setup_interrupt, 
    .powerup            = avmnet_ar803x_powerup,         
    .powerdown          = avmnet_ar803x_powerdown,       
    .suspend            = avmnet_ar803x_suspend,         
    .resume             = avmnet_ar803x_resume,          
    .reinit             = avmnet_ar803x_reinit,

    .ethtool_ops    = AR803X_ETHOPS,

    .parent         = &hw214_gmac0,
    .num_children   = 0,
    .children       = {}
};


#endif /*--- #if !defined(__AVM_NET_CFG_ATHEROS_HW200_H_) ---*/
